@@ -778,6 +778,7 @@ void KUrlTest::testPrettyURL()
   QCOMPARE( urlWithPassAndNoUser.prettyUrl(), QString::fromLatin1( "ftp://ftp.kde.org/path" ) );
 
   KUrl xmppUri("xmpp:ogoffart@kde.org");
+  QCOMPARE(xmppUri.path(), QString::fromLatin1("ogoffart@kde.org"));
   QCOMPARE( xmppUri.prettyUrl(), QString::fromLatin1( "xmpp:ogoffart@kde.org" ) );
 
   QUrl offEagleqUrl;
@@ -923,8 +924,13 @@ void KUrlTest::testAdjustPath()
 
     {
     KUrl remote2("remote://");
-    QCOMPARE( remote2.url(), QString("remote:") );
-    QCOMPARE( remote2.url(KUrl::RemoveTrailingSlash ), QString("remote:") );
+#if QT_VERSION < 0x040805
+    QCOMPARE( remote2.url(), QString("remote:") ); // QUrl bug, fixed in Qt 4.8.5 and Qt5
+#else
+    QCOMPARE( remote2.url(), QString("remote://") );
+#endif
+    QCOMPARE( remote2.url(KUrl::RemoveTrailingSlash ), QString("remote:") ); // QUrl bug, fixed in Qt5
+    QCOMPARE( remote2.prettyUrl(), QString("remote://") );
     }
 }
 
@@ -1712,12 +1718,28 @@ void KUrlTest::testSmb()
   QCOMPARE( smb.user(), QString("domain;username") );
   smb = "smb:/";
   QVERIFY( smb.isValid() );
+  QCOMPARE(smb.url(), QString::fromLatin1("smb:/"));
+  QCOMPARE(smb.prettyUrl(), QString::fromLatin1("smb:/"));
   smb = "smb://"; // KDE3: kurl.cpp rev 1.106 made it invalid. Valid again with QUrl.
   QVERIFY( smb.isValid() );
+#if QT_VERSION < 0x040805
+  QCOMPARE(smb.url(), QString::fromLatin1("smb:")); // QUrl bug, fixed in Qt 4.8.5 and Qt5
+#else
+  QCOMPARE(smb.url(), QString::fromLatin1("smb://"));
+#endif
+  QCOMPARE(smb.prettyUrl(), QString::fromLatin1("smb://"));
   smb = "smb://host";
   QVERIFY( smb.isValid() );
+  QCOMPARE(smb.url(), QString::fromLatin1("smb://host"));
+  QCOMPARE(smb.prettyUrl(), QString::fromLatin1("smb://host"));
   smb = "smb:///";
   QVERIFY( smb.isValid() );
+#if QT_VERSION < 0x040805
+  QCOMPARE(smb.url(), QString::fromLatin1("smb:/")); // QUrl bug, fixed in Qt 4.8.5 and Qt5
+#else
+  QCOMPARE(smb.url(), QString::fromLatin1("smb:///"));
+#endif
+  QCOMPARE(smb.prettyUrl(), QString::fromLatin1("smb:/"));
 
   KUrl implicitSmb("file://host/path");
   QVERIFY(!implicitSmb.isLocalFile()); // -> kio_file will redirect to smb (by default)
