@@ -1307,13 +1307,18 @@ bool KFileItem::cmp( const KFileItem & item ) const
 
 bool KFileItem::operator==(const KFileItem& other) const
 {
-    // is this enough?
-    return d == other.d;
+    if (!d && !other.d)
+        return true;
+
+    if (!d || !other.d)
+        return false;
+
+    return d->m_url == other.d->m_url;
 }
 
 bool KFileItem::operator!=(const KFileItem& other) const
 {
-    return d != other.d;
+    return !operator==(other);
 }
 
 #ifndef KDE_NO_DEPRECATED
