@@ -304,7 +304,7 @@ BrowserRun::NonEmbeddableResult BrowserRun::handleNonEmbeddable(const QString& _
                          this, SLOT(slotCopyToTempFileResult(KJob*)) );
                 return Delayed; // We'll continue after the job has finished
             }
-            if (selectedService) {
+            if (selectedService && question.selectedService()) {
                 *selectedService = question.selectedService();
                 // KRun will use this when starting an app
                 KRun::setPreferredService(question.selectedService()->desktopEntryName());
