@@ -19,6 +19,7 @@
 
 #include "dataengine.h"
 #include "private/dataengine_p.h"
+#include "private/datacontainer_p.h"
 
 #include <QQueue>
 #include <QTimer>
@@ -41,19 +42,20 @@
 #include "private/dataengineservice_p.h"
 #include "private/remotedataengine_p.h"
 #include "private/service_p.h"
+#include "private/storage_p.h"
 
 namespace Plasma
 {
 
 DataEngine::DataEngine(QObject *parent, KService::Ptr service)
     : QObject(parent),
-      d(new DataEnginePrivate(this, service))
+      d(new DataEnginePrivate(this, KPluginInfo(service)))
 {
 }
 
 DataEngine::DataEngine(QObject *parent, const QVariantList &args)
     : QObject(parent),
-      d(new DataEnginePrivate(this, KService::serviceByStorageId(args.count() > 0 ? args[0].toString() : QString())))
+      d(new DataEnginePrivate(this, KPluginInfo(KService::serviceByStorageId(args.count() > 0 ?  args[0].toString() : QString()))))
 {
 }
 
@@ -195,7 +197,7 @@ void DataEngine::setData(const QString &source, const QString &key, const QVaria
 
     s->setData(key, value);
 
-    if (isNew) {
+    if (isNew && source != d->waitingSourceRequest) {
         emit sourceAdded(source);
     }
 
@@ -217,7 +219,7 @@ void DataEngine::setData(const QString &source, const Data &data)
         ++it;
     }
 
-    if (isNew) {
+    if (isNew && source != d->waitingSourceRequest) {
         emit sourceAdded(source);
     }
 
@@ -251,6 +253,7 @@ void DataEngine::addSource(DataContainer *source)
 
     QObject::connect(source, SIGNAL(updateRequested(DataContainer*)),
                      this, SLOT(internalUpdateSource(DataContainer*)));
+    QObject::connect(source, SIGNAL(destroyed(QObject*)), this, SLOT(sourceDestroyed(QObject*)));
     d->sources.insert(source->objectName(), source);
     emit sourceAdded(source->objectName());
     scheduleSourcesUpdated();
@@ -278,10 +281,6 @@ uint DataEngine::maxSourceCount() const
 
 void DataEngine::setMinimumPollingInterval(int minimumMs)
 {
-    if (minimumMs < 0) {
-        minimumMs = 0;
-    }
-
     d->minPollingInterval = minimumMs;
 }
 
@@ -300,20 +299,8 @@ void DataEngine::setPollingInterval(uint frequency)
     }
 }
 
-/*
-NOTE: This is not implemented to prevent having to store the value internally.
-      When there is a good use case for needing access to this value, we can
-      add another member to the Private class and add this method.
-
-void DataEngine::pollingInterval()
-{
-    return d->pollingInterval;
-}
-*/
-
 void DataEngine::removeSource(const QString &source)
 {
-    //kDebug() << "removing source " << source;
     SourceDict::iterator it = d->sources.find(source);
     if (it != d->sources.end()) {
         DataContainer *s = it.value();
@@ -330,6 +317,8 @@ void DataEngine::removeSource(const QString &source)
             }
         }
 
+        s->d->store();
+        s->disconnect(this);
         s->deleteLater();
         d->sources.erase(it);
         emit sourceRemoved(source);
@@ -341,9 +330,12 @@ void DataEngine::removeAllSources()
     QMutableHashIterator<QString, Plasma::DataContainer*> it(d->sources);
     while (it.hasNext()) {
         it.next();
-        emit sourceRemoved(it.key());
-        delete it.value();
+        Plasma::DataContainer *s = it.value();
+        const QString source = it.key();
         it.remove();
+        s->disconnect(this);
+        s->deleteLater();
+        emit sourceRemoved(source);
     }
 }
 
@@ -384,7 +376,6 @@ void DataEngine::timerEvent(QTimerEvent *event)
         }
 
         d->updateTimestamp.restart();
-        d->updateTimerId = 0;
         updateAllSources();
     } else if (event->timerId() == d->checkSourcesTimerId) {
         killTimer(d->checkSourcesTimerId);
@@ -438,6 +429,18 @@ QString DataEngine::pluginName() const
     return d->dataEngineDescription.pluginName();
 }
 
+void DataEngine::setDefaultService(const QString &serviceName)
+{
+    d->serviceName = serviceName;
+}
+
+Service* DataEngine::createDefaultService(QObject *parent)
+{
+    QVariantList args;
+    args << QVariant::fromValue<DataEngine*>(this);
+    return Service::load(d->serviceName, args, parent);
+}
+
 void DataEnginePrivate::publish(AnnouncementMethods methods, const QString &name)
 {
     if (!publishedService) {
@@ -495,10 +498,18 @@ void DataEngine::setName(const QString &name)
     setObjectName(name);
 }
 
+void DataEngine::setStorageEnabled(const QString &source, bool store)
+{
+    DataContainer *s = d->source(source, false);
+    if (s) {
+        s->setStorageEnabled(store);
+    }
+}
+
 // Private class implementations
-DataEnginePrivate::DataEnginePrivate(DataEngine *e, KService::Ptr service)
+DataEnginePrivate::DataEnginePrivate(DataEngine *e, const KPluginInfo &info)
     : q(e),
-      dataEngineDescription(service),
+      dataEngineDescription(info),
       refCount(-1), // first ref
       checkSourcesTimerId(0),
       updateTimerId(0),
@@ -511,17 +522,17 @@ DataEnginePrivate::DataEnginePrivate(DataEngine *e, KService::Ptr service)
 {
     updateTimestamp.start();
 
-    if (!service) {
+    if (!info.isValid()) {
         engineName = i18n("Unnamed");
         return;
     }
 
-    engineName = service->name();
+    engineName = info.name();
     if (engineName.isEmpty()) {
         engineName = i18n("Unnamed");
     }
     e->setObjectName(engineName);
-    icon = service->icon();
+    icon = info.icon();
 
     if (dataEngineDescription.isValid()) {
         QString api = dataEngineDescription.property("X-Plasma-API").toString();
@@ -530,8 +541,7 @@ DataEnginePrivate::DataEnginePrivate(DataEngine *e, KService::Ptr service)
             const QString path =
                 KStandardDirs::locate("data",
                                       "plasma/dataengines/" + dataEngineDescription.pluginName() + '/');
-            PackageStructure::Ptr structure =
-                Plasma::packageStructure(api, Plasma::DataEngineComponent);
+            PackageStructure::Ptr structure = Plasma::packageStructure(api, Plasma::DataEngineComponent);
             structure->setPath(path);
             package = new Package(path, structure);
 
@@ -606,19 +616,18 @@ DataContainer *DataEnginePrivate::source(const QString &sourceName, bool createW
             }
             sourceQueue.enqueue(s);
         }
-        return it.value();
+        return s;
     }
 
     if (!createWhenMissing) {
         return 0;
     }
 
-    /*kDebug() << "DataEngine " << q->objectName()
-                << ": could not find DataContainer " << sourceName
-                << ", creating" << endl;*/
+    //kDebug() << "DataEngine " << q->objectName() << ": could not find DataContainer " << sourceName << ", creating";
     DataContainer *s = new DataContainer(q);
     s->setObjectName(sourceName);
     sources.insert(sourceName, s);
+    QObject::connect(s, SIGNAL(destroyed(QObject*)), q, SLOT(sourceDestroyed(QObject*)));
     QObject::connect(s, SIGNAL(updateRequested(DataContainer*)),
                      q, SLOT(internalUpdateSource(DataContainer*)));
 
@@ -663,6 +672,20 @@ void DataEnginePrivate::connectSource(DataContainer *s, QObject *visualization,
         QMetaObject::invokeMethod(visualization, "dataUpdated",
                                   Q_ARG(QString, s->objectName()),
                                   Q_ARG(Plasma::DataEngine::Data, s->data()));
+        s->d->dirty = false;
+    }
+}
+
+void DataEnginePrivate::sourceDestroyed(QObject *object)
+{
+    DataEngine::SourceDict::iterator it = sources.begin();
+    while (it != sources.end()) {
+        if (it.value() == object) {
+            sources.erase(it);
+            emit q->sourceRemoved(object->objectName());
+            break;
+        }
+        ++it;
     }
 }
 
@@ -680,6 +703,7 @@ DataContainer *DataEnginePrivate::requestSource(const QString &sourceName, bool
         /*kDebug() << "DataEngine " << q->objectName()
             << ": could not find DataContainer " << sourceName
             << " will create on request" << endl;*/
+        waitingSourceRequest = sourceName;
         if (q->sourceRequestEvent(sourceName)) {
             s = source(sourceName, false);
             if (s) {
@@ -689,8 +713,10 @@ DataContainer *DataEnginePrivate::requestSource(const QString &sourceName, bool
                     *newSource = true;
                 }
                 QObject::connect(s, SIGNAL(becameUnused(QString)), q, SLOT(removeSource(QString)));
+                emit q->sourceAdded(sourceName);
             }
         }
+        waitingSourceRequest.clear();
     }
 
     return s;
@@ -714,9 +740,11 @@ void DataEnginePrivate::setupScriptSupport()
         return;
     }
 
-    kDebug() << "setting up script support, package is in" << package->path()
+    /*
+    kDebug() << "sletting up script support, package is in" << package->path()
              << "which is a" << package->structure()->type() << "package"
              << ", main script is" << package->filePath("mainscript");
+    */
 
     QString translationsPath = package->filePath("translations");
     if (!translationsPath.isEmpty()) {
