@@ -37,25 +37,22 @@ Battery::~Battery()
 
 bool Battery::isPlugged() const
 {
-    return m_device->prop("IsPresent").toBool();
+    return m_device.data()->prop("IsPresent").toBool();
 }
 
 Solid::Battery::BatteryType Battery::type() const
 {
     Solid::Battery::BatteryType result = Solid::Battery::UnknownBattery;
-    const uint t = m_device->prop("Type").toUInt();
-    bool powerSupply = m_device->prop("PowerSupply").toBool();
+    const uint t = m_device.data()->prop("Type").toUInt();
     switch (t)
     {
         case 1: // TODO "Line Power"
             break;
         case 2:
-            if (powerSupply)
-                result = Solid::Battery::PrimaryBattery;
+            result = Solid::Battery::PrimaryBattery;
             break;
         case 3:
-            if (powerSupply)
-                result = Solid::Battery::UpsBattery;
+            result = Solid::Battery::UpsBattery;
             break;
         case 4:
             result = Solid::Battery::MonitorBattery;
@@ -78,18 +75,18 @@ Solid::Battery::BatteryType Battery::type() const
 
 int Battery::chargePercent() const
 {
-    return qRound(m_device->prop("Percentage").toDouble());
+    return qRound(m_device.data()->prop("Percentage").toDouble());
 }
 
 bool Battery::isRechargeable() const
 {
-    return m_device->prop("IsRechargeable").toBool();
+    return m_device.data()->prop("IsRechargeable").toBool();
 }
 
 Solid::Battery::ChargeState Battery::chargeState() const
 {
     Solid::Battery::ChargeState result = Solid::Battery::NoCharge;
-    const uint state = m_device->prop("State").toUInt();
+    const uint state = m_device.data()->prop("State").toUInt();
     switch (state)
     {
         case 0:
@@ -115,27 +112,26 @@ Solid::Battery::ChargeState Battery::chargeState() const
 
 void Battery::slotChanged()
 {
-    if (!m_device)
-        return;
+    if (m_device) {
+        const bool old_isPlugged = m_isPlugged;
+        const int old_chargePercent = m_chargePercent;
+        const Solid::Battery::ChargeState old_chargeState = m_chargeState;
+        updateCache();
 
-    const bool old_isPlugged = m_isPlugged;
-    const int old_chargePercent = m_chargePercent;
-    const Solid::Battery::ChargeState old_chargeState = m_chargeState;
-    updateCache();
+        if (old_chargePercent != m_chargePercent)
+        {
+            emit chargePercentChanged(m_chargePercent, m_device.data()->udi());
+        }
 
-    if (old_chargePercent != m_chargePercent)
-    {
-        emit chargePercentChanged(m_chargePercent, m_device->udi());
-    }
+        if (old_chargeState != m_chargeState)
+        {
+            emit chargeStateChanged(m_chargeState, m_device.data()->udi());
+        }
 
-    if (old_chargeState != m_chargeState)
-    {
-        emit chargeStateChanged(m_chargeState, m_device->udi());
-    }
-
-    if (old_isPlugged != m_isPlugged)
-    {
-        emit plugStateChanged(m_isPlugged, m_device->udi());
+        if (old_isPlugged != m_isPlugged)
+        {
+            emit plugStateChanged(m_isPlugged, m_device.data()->udi());
+        }
     }
 }
 
